local addonName = ...
local NameplateHighlight = CreateFrame('Frame')
NameplateHighlight.unitCache = {}

local Specials = {
	[179912] = true, -- Maelie the Wanderer
	[162681] = true, -- Elusive Quickhoof
	[162765] = true, -- Friendly Alpaca
	[65090] = true, -- Fabious
}

local tooltip = CreateFrame("GameTooltip",("%sTooltip"):format("NameplateHighlight"), UIParent, "GameTooltipTemplate");
local function GetCreatureNameFromID(id)
	tooltip:SetOwner(UIParent, "ANCHOR_NONE");
	tooltip:SetHyperlink(("unit:Creature-0-0-0-0-%d"):format(id))
	local line = _G[tooltip:GetName().."TextLeft1"]:GetText()
	tooltip:ClearLines()
	tooltip:Hide()
	return line
end

local OverlayPool = CreateFramePool("PlayerModel", WorldFrame, "NameplateHighlightOverlayTemplate")
local Overlay = {}

function Overlay:ShowOverlay(unit)
	local namePlate = C_NamePlate.GetNamePlateForUnit(unit)
	if namePlate then
		local overlay = OverlayPool:Acquire()
		overlay:SetModel(948186)
		overlay:ClearAllPoints()
		overlay:SetPoint("BOTTOM", namePlate, "TOP", 0, -5)
		overlay:Show()
		overlay.Anim:Play()
		Overlay[namePlate] = overlay
	end
end

function Overlay:HideOverlay(unit)
	local namePlate = C_NamePlate.GetNamePlateForUnit(unit)
	if OverlayPool:IsActive(Overlay[namePlate]) then
		OverlayPool:Release(Overlay[namePlate])
	end
end

function NameplateHighlight:OnEvent(e, ...)
	if e == "ADDON_LOADED" and ... == addonName then
		NameplateHighlightDB = NameplateHighlightDB or {}
		NameplateHighlightDB.Specials = NameplateHighlightDB.Specials or {}
	elseif e == "NAME_PLATE_UNIT_ADDED" then
		local unit = ...
		if not unit then return end
		local npcID=tonumber((select(6,strsplit("-", UnitGUID(unit)))))
		if UnitCanAttack("player",unit) or Specials[npcID] then
			local unitClassification = UnitClassification(unit)
			if unitClassification == "rareelite" or unitClassification == "rare" then
				self.unitCache[unit] = true
				Overlay:ShowOverlay(unit)
				return
			end
		end

		if Specials[npcID] or NameplateHighlightDB.Specials[npcID] then
			self.unitCache[unit] = true
			Overlay:ShowOverlay(unit)
		end
	elseif e == "NAME_PLATE_UNIT_REMOVED" then
		local unit = ...
		if not unit then return end
        if self.unitCache[unit] then
            self.unitCache[unit] = nil
			Overlay:HideOverlay(unit)
		end
	end
end

function NameplateHighlight:ContinueOrDelay(exists, ContinueFunc)
	if exists then
		ContinueFunc()
		return
	end

	local latency = ((select(4, GetNetStats()) + 20) / 100)
	C_Timer.After(latency, function()
		ContinueFunc()
	end)
end

function NameplateHighlight:Help(msg)
	local fName = "|cffEEE4AENameplate Highlight:|r "
	local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")
	if not cmd or cmd == "" or cmd == "help" then
		print("|cffEEE4AENameplate Highlight: /nph /nameplatehighlight|r")
		print("   Add Custom NPC ID (or current target without id) - /nph add ID")
		print("   Remove Custom NPC ID (or current target without id) - /nph rem ID")
		print("   Remove All Custom NPC IDs - /nph wipe")
	elseif cmd == "add" then
		local ADD = function(NPCID)
			if NPCID then
				local CreatureName = GetCreatureNameFromID(NPCID)
				self:ContinueOrDelay(CreatureName, function()
					CreatureName = GetCreatureNameFromID(NPCID) or "Unknown"
					if (Specials[NPCID] or NameplateHighlightDB.Specials[NPCID]) then
						print(fName..CreatureName.." ("..NPCID..")|r already exists.")
						return
					end
					NameplateHighlightDB.Specials[NPCID] = true
					print(fName.."|cff37DB33Adding|r "..CreatureName.." ("..NPCID..")")
				end)
			end
		end

		if args == "" then
			local unit = UnitGUID("target")
			if unit then
			local NPCID = tonumber((select(6,strsplit("-", unit))))
				ADD(NPCID)
			else
				print(fName.."Nothing to add")
			end
		else
			local NPCID = tonumber(args:match("%d+"))
			ADD(NPCID)
		end
	elseif (cmd == "rem" or cmd == "remove") then
		local REMOVE = function(NPCID)
			if NPCID and NameplateHighlightDB.Specials[NPCID] then
				local CreatureName = GetCreatureNameFromID(NPCID)
				self:ContinueOrDelay(CreatureName, function()
					CreatureName = GetCreatureNameFromID(NPCID) or "Unknown"
					NameplateHighlightDB.Specials[NPCID] = nil
					print(fName.."|cffFF4646Removed|r "..CreatureName.." ("..NPCID..")")
				end)
			else
				print(fName.."Nothing to remove")
			end
		end
		if args == "" then
			local unit = UnitGUID("target")
			if unit then
				local NPCID = tonumber((select(6,strsplit("-", unit))))
				REMOVE(NPCID)
			else
				print(fName.."Nothing to remove")
			end
		else
			local NPCID = tonumber(args:match("%d+"))
			REMOVE(NPCID)
		end
	elseif cmd == "wipe" or cmd == "clear" then
		wipe(NameplateHighlightDB.Specials)
		print(fName.."|cff37DB33Removed all previously added NPC's|r")
	end
end

SLASH_NAMEPLATEHIGHLIGHT1, SLASH_NAMEPLATEHIGHLIGHT2 = '/nph', '/nameplatehighlight'
SlashCmdList["NAMEPLATEHIGHLIGHT"] = function(...)
    NameplateHighlight:Help(...)
end

local function RegisterForEvents(self, e) for i=1, #e do self:RegisterEvent(e[i]) end end
NameplateHighlight:SetScript("OnEvent", NameplateHighlight.OnEvent)
RegisterForEvents(NameplateHighlight, {
	"ADDON_LOADED",
	"NAME_PLATE_UNIT_ADDED",
	"NAME_PLATE_UNIT_REMOVED"
})